#ifndef MatricesH //To avoid linker errors, define header's unique "name", so...
#define MatricesH //...this header will not be compilled more than one time
//==============================================================================

//Macros for calculating determinant of 2x2 square matrix (3 basic operations)
#define Determinant2x2(M) (M[0]*M[3]-M[1]*M[2])
//Macros for calculating determinant of 3x3 square matrix (14 basic operations)
#define Determinant3x3(M) (M[0]*(M[4]*M[8]-M[5]*M[7])-M[3]*(M[1]*M[8]-M[2]*M[7])+M[6]*(M[1]*M[5]-M[2]*M[4]))
//Macros for calculating determinant of 3x3 matrix composed from 3 vectors (14 basic operations)
#define Determinant3xV(a,b,c) (a[0]*(b[1]*c[2]-b[2]*c[1])+a[1]*(b[2]*c[0]-b[0]*c[2])+a[2]*(b[0]*c[1]-b[1]*c[0]))
//Macros for calculating determinant of 4x4 square matrix (63 basic operations)
#define Determinant4x4(M) (M[0]*(M[5]*(M[10]*M[15]-M[11]*M[14])-M[9]*(M[6]*M[15]-M[7]*M[14])+M[13]*(M[6]*M[11]-M[7]*M[10]))-M[4]*(M[1]*(M[10]*M[15]-M[11]*M[14])-M[9]*(M[2]*M[15]-M[3]*M[14])+M[13]*(M[2]*M[11]-M[3]*M[10]))+M[8]*(M[1]*(M[6]*M[15]-M[7]*M[14])-M[5]*(M[2]*M[15]-M[3]*M[14])+M[13]*(M[2]*M[7]-M[3]*M[6]))-M[12]*(M[1]*(M[6]*M[11]-M[7]*M[10])-M[5]*(M[2]*M[11]-M[3]*M[10])+M[9]*(M[2]*M[7]-M[3]*M[6])))

//Function summs two matrixes
inline void SummMatrixes2x2(const float A[4], const float B[4], float *S){
 S[0] = A[0]+B[0];
 S[1] = A[1]+B[1];
 S[2] = A[2]+B[2];
 S[3] = A[3]+B[3];
}
inline void SummMatrixes3x3(const float A[9], const float B[9], float *S){
 S[0] = A[0]+B[0];
 S[1] = A[1]+B[1];
 S[2] = A[2]+B[2];
 S[3] = A[3]+B[3];
 S[4] = A[4]+B[4];
 S[5] = A[5]+B[5];
 S[6] = A[6]+B[6];
 S[7] = A[7]+B[7];
 S[8] = A[8]+B[8];
}
inline void SummMatrixes4x4(const float A[16], const float B[16], float *S){
 S[0] = A[0]+B[0];
 S[1] = A[1]+B[1];
 S[2] = A[2]+B[2];
 S[3] = A[3]+B[3];
 S[4] = A[4]+B[4];
 S[5] = A[5]+B[5];
 S[6] = A[6]+B[6];
 S[7] = A[7]+B[7];
 S[8] = A[8]+B[8];
 S[9] = A[9]+B[9];
 S[10] = A[10]+B[10];
 S[11] = A[11]+B[11];
 S[12] = A[12]+B[12];
 S[13] = A[13]+B[13];
 S[14] = A[14]+B[14];
 S[15] = A[15]+B[15];
}

//Functions multiply matrix A to matrix B
inline void MultMatrixes2x2(const float A[4], const float B[4], float *M){
 M[0] = A[0]*B[0] + A[2]*B[1];
 M[1] = A[1]*B[0] + A[3]*B[1];
 M[2] = A[0]*B[2] + A[2]*B[3];
 M[3] = A[1]*B[2] + A[3]*B[3];
}
inline void MultMatrixes3x3(const float A[9], const float B[9], float *M){
 M[0] = A[0]*B[0] + A[3]*B[1] + A[6]*B[2];
 M[1] = A[1]*B[0] + A[4]*B[1] + A[7]*B[2];
 M[2] = A[2]*B[0] + A[5]*B[1] + A[8]*B[2];
 M[3] = A[0]*B[3] + A[3]*B[4] + A[6]*B[5];
 M[4] = A[1]*B[3] + A[4]*B[4] + A[7]*B[5];
 M[5] = A[2]*B[3] + A[5]*B[4] + A[8]*B[5];
 M[6] = A[0]*B[6] + A[3]*B[7] + A[6]*B[8];
 M[7] = A[1]*B[6] + A[4]*B[7] + A[7]*B[8];
 M[8] = A[2]*B[6] + A[5]*B[7] + A[8]*B[8];
}
inline void MultMatrixes4x4(const float A[16], const float B[16], float *M){
 M[0] = A[0]*B[0] + A[4]*B[1] +  A[8]*B[2] + A[12]*B[3];
 M[1] = A[1]*B[0] + A[5]*B[1] +  A[9]*B[2] + A[13]*B[3];
 M[2] = A[2]*B[0] + A[6]*B[1] + A[10]*B[2] + A[14]*B[3];
 M[3] = A[3]*B[0] + A[7]*B[1] + A[11]*B[2] + A[15]*B[3];
 M[4] = A[0]*B[4] + A[4]*B[5] +  A[8]*B[6] + A[12]*B[7];
 M[5] = A[1]*B[4] + A[5]*B[5] +  A[9]*B[6] + A[13]*B[7];
 M[6] = A[2]*B[4] + A[6]*B[5] + A[10]*B[6] + A[14]*B[7];
 M[7] = A[3]*B[4] + A[7]*B[5] + A[11]*B[6] + A[15]*B[7];
 M[8] = A[0]*B[8] + A[4]*B[9] +  A[8]*B[10] + A[12]*B[11];
 M[9] = A[1]*B[8] + A[5]*B[9] +  A[9]*B[10] + A[13]*B[11];
 M[10] = A[2]*B[8] + A[6]*B[9] + A[10]*B[10] + A[14]*B[11];
 M[11] = A[3]*B[8] + A[7]*B[9] + A[11]*B[10] + A[15]*B[11];
 M[12] = A[0]*B[12] + A[4]*B[13] +  A[8]*B[14] + A[12]*B[15];
 M[13] = A[1]*B[12] + A[5]*B[13] +  A[9]*B[14] + A[13]*B[15];
 M[14] = A[2]*B[12] + A[6]*B[13] + A[10]*B[14] + A[14]*B[15];
 M[15] = A[3]*B[12] + A[7]*B[13] + A[11]*B[14] + A[15]*B[15];
}

//Functions calculate invert matrix for matrix A
inline void InvertMatrix2x2(const float A[4], float *I){
 float DetA = Determinant2x2(A);
 I[0] =  A[3]/DetA;
 I[1] = -A[1]/DetA;
 I[2] = -A[2]/DetA;
 I[3] =  A[0]/DetA;
}
inline void InvertMatrix3x3(const float A[9], float *I){
 float DetA = Determinant3x3(A);
 I[0] = (A[4]*A[8]-A[5]*A[7])/DetA;
 I[1] = (A[2]*A[7]-A[1]*A[8])/DetA;
 I[2] = (A[1]*A[5]-A[2]*A[4])/DetA;
 I[3] = (A[5]*A[6]-A[3]*A[8])/DetA;
 I[4] = (A[0]*A[8]-A[2]*A[6])/DetA;
 I[5] = (A[2]*A[3]-A[0]*A[5])/DetA;
 I[6] = (A[3]*A[7]-A[4]*A[6])/DetA;
 I[7] = (A[1]*A[6]-A[0]*A[7])/DetA;
 I[8] = (A[0]*A[4]-A[1]*A[3])/DetA;
}
#define Determinant9(M0,M1,M2,M3,M4,M5,M6,M7,M8) (M0*(M4*M8-M5*M7)-M3*(M1*M8-M2*M7)+M6*(M1*M5-M2*M4))
inline void InvertMatrix4x4(const float A[16], float *I){
 float DetA = Determinant4x4(A);
 I[0] = Determinant9(A[5],A[6],A[7],A[9],A[10],A[11],A[13],A[14],A[15])/DetA;
 I[1] = Determinant9(A[9],A[10],A[11],A[1],A[2],A[3],A[13],A[14],A[15])/DetA;
 I[2] = Determinant9(A[1],A[2],A[3],A[5],A[6],A[7],A[13],A[14],A[15])/DetA;
 I[3] = Determinant9(A[5],A[6],A[7],A[1],A[2],A[3],A[9],A[10],A[11])/DetA;
 I[4] = Determinant9(A[8],A[10],A[11],A[4],A[6],A[7],A[12],A[14],A[15])/DetA;
 I[5] = Determinant9(A[0],A[2],A[3],A[8],A[10],A[11],A[12],A[14],A[15])/DetA;
 I[6] = Determinant9(A[4],A[6],A[7],A[0],A[2],A[3],A[12],A[14],A[15])/DetA;
 I[7] = Determinant9(A[0],A[2],A[3],A[4],A[6],A[7],A[8],A[10],A[11])/DetA;
 I[8] = Determinant9(A[4],A[5],A[7],A[8],A[9],A[11],A[12],A[13],A[15])/DetA;
 I[9] = Determinant9(A[8],A[9],A[11],A[0],A[1],A[3],A[12],A[13],A[15])/DetA;
 I[10] = Determinant9(A[0],A[1],A[3],A[4],A[5],A[7],A[12],A[13],A[15])/DetA;
 I[11] = Determinant9(A[4],A[5],A[7],A[0],A[1],A[3],A[8],A[9],A[11])/DetA;
 I[12] = Determinant9(A[8],A[9],A[10],A[4],A[5],A[6],A[12],A[13],A[14])/DetA;
 I[13] = Determinant9(A[0],A[1],A[2],A[8],A[9],A[10],A[12],A[13],A[14])/DetA;
 I[14] = Determinant9(A[4],A[5],A[6],A[0],A[1],A[2],A[12],A[13],A[14])/DetA;
 I[15] = Determinant9(A[0],A[1],A[2],A[4],A[5],A[6],A[8],A[9],A[10])/DetA;
}

//Functions calculate transpose matrix for matrix A
inline void TransposeMatrix2x2(const float A[4], float *T){
 T[0]=A[0];
 T[1]=A[2];
 T[2]=A[1];
 T[3]=A[3];
}
inline void TransposeMatrix3x3(const float A[9], float *T){
 T[0]=A[0];
 T[1]=A[3];
 T[2]=A[6];
 T[3]=A[1];
 T[4]=A[4];
 T[5]=A[7];
 T[6]=A[2];
 T[7]=A[5];
 T[8]=A[8];
}
inline void TransposeMatrix4x4(const float A[16], float *T){
 T[0]=A[0];
 T[1]=A[4];
 T[2]=A[8];
 T[3]=A[12];
 T[4]=A[1];
 T[5]=A[5];
 T[6]=A[9];
 T[7]=A[13];
 T[8]=A[2];
 T[9]=A[6];
 T[10]=A[10];
 T[11]=A[14];
 T[12]=A[3];
 T[13]=A[7];
 T[14]=A[11];
 T[15]=A[15];
}
//==============================================================================
#endif //End of header